<?php
/**
 * @package Site AB Tester
 * @author Rick Wash
 * @version 0.1
 */
/*
   Plugin Name: Site AB Tester
   Plugin URI: http://wordpress.org/#
   Description: This plugin allows you to conduct a site-wide A/B test
   Author: Rick Wash
   Version: 0.1
   Author URI: http://www.rickwash.com
   */
   
/*******
 Code to initialize the plugin
 - Creates options
 - Creates the logging database table
 *******/
   
function create_siteab_options() {
  add_option('siteab_next_user_id', 1);
  add_option('siteab_experiment', 'disabled');
  add_option('siteab_debug', 'disabled');
  add_option('siteab_debug_force_condition', '0');
  add_option('siteab_debug_condition', 'disabled');
  add_option('siteab_enable_comment_ratings', 0);
}
register_activation_hook(__FILE__, 'create_siteab_options');

/** Code for creating a database table or two **/
function siteab_init_db() {
  global $wpdb;

  $siteab_db_current_version = 6;
  
  $log_table_name = $wpdb->prefix . "siteab_log";
  $log_sql = "CREATE TABLE " . $log_table_name . " (
      id integer NOT NULL AUTO_INCREMENT,
      time timestamp DEFAULT current_timestamp NOT NULL,
      userid integer NOT NULL,
      experiment varchar(50) NOT NULL,
      cond varchar(50) NOT NULL,
      action varchar(200) DEFAULT NULL,
      postid integer DEFAULT NULL,
      commentid integer DEFAULT NULL,
      PRIMARY KEY id (id)
    );";
  $rating_table_name = $wpdb->prefix . "siteab_ratings";
  $rating_sql = "CREATE TABLE " . $rating_table_name . " (
      id integer NOT NULL AUTO_INCREMENT,
      userid integer NOT NULL,
      postid integer NOT NULL,
      cond varchar(50) NOT NULL,
      rating integer NOT NULL,
      PRIMARY KEY id (id)
    );";
  $comment_rating_table_name = $wpdb->prefix . "siteab_comment_ratings";
  $comment_rating_sql = "CREATE TABLE " . $comment_rating_table_name . " (
      id integer NOT NULL AUTO_INCREMENT,
      userid integer NOT NULL,
      postid integer NOT NULL,
      commentid integer NOT NULL,
      cond varchar(50) NOT NULL,
      rating integer NOT NULL,
      PRIMARY KEY id (id)
    );";  
  if ($wpdb->get_var("SHOW TABLES LIKE '$log_table_name'") != $log_table_name) {
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($log_sql);
//    $wpdb->query($log_sql);
    add_option("siteab_db_version", $siteab_db_current_version);
  }
  if ($wpdb->get_var("SHOW TABLES LIKE '$rating_table_name'") != $rating_table_name) {
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($rating_sql);
//    $wpdb->query($rating_sql);
    add_option("siteab_db_version", $siteab_db_current_version);
  }
  if ($wpdb->get_var("SHOW TABLES LIKE '$comment_rating_table_name'") != $rating_table_name) {
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($comment_rating_sql);
//    $wpdb->query($rating_sql);
    add_option("siteab_db_version", $siteab_db_current_version);
  }
  
  // XXX: Note, this destroys the old log table when there is a new table schema
  $installed_ver = get_option("siteab_db_version");
  if ($installed_ver != $siteab_db_current_version) {
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($log_sql);
    dbDelta($rating_sql);
    dbDelta($comment_rating_sql);
//     $wpdb->query('drop table ' . $log_table_name);
//     $wpdb->query('drop table ' . $rating_table_name);
//     $wpdb->query($log_sql);
//     $wpdb->query($rating_sql);
     update_option("siteab_db_version", $siteab_db_current_version);
  }
}

register_activation_hook(__FILE__, "siteab_init_db");

/********
 Debugging code.   Records messages and appends them to the end of the page
 ********/
$siteab_debug_msgs = "";
function siteab_debug($msg) {
  global $siteab_debug_msgs;
  
  $siteab_debug_msgs = $siteab_debug_msgs . '<p>' . $msg . '</p>';
}

function siteab_print_debug_msgs() {
  global $siteab_debug_msgs;
  
  echo '<div class="debug"><span style="font-weight: bold;">Debugging Messages</span>';
  echo $siteab_debug_msgs;
  echo '</div>';
}

function siteab_show_errors() {
  global $wpdb;
  $wpdb->show_errors();
}

if ((get_option('siteab_debug') == 'enabled') && (!is_admin())) {
  add_action('init', 'siteab_show_errors');
  add_action('wp_footer', 'siteab_print_debug_msgs');
}

/********
 Main code that gets run for every page.
 - Determines which condition the user is in
 - Assigns the user a condition if they don't have one already
 - Logs every page load to the database
 ********/

// ** Constant names for each condition: Comment experiment
define("siteab_comment_noask", "No Ask");
define("siteab_comment_com_load", "Comment-onLoad");
define("siteab_comment_com_scroll", "Comment-onScroll");
define("siteab_comment_later_load", "Later-onLoad");
define("siteab_comment_later_scroll", "Later-onScroll");

// ** Constant names for each condition: Rating experiment
define("siteab_rating_noask", "No Ask");
define("siteab_rating_com_load", "Rate-onLoad");
define("siteab_rating_com_scroll", "Rate-onScroll");
define("siteab_rating_later_load", "Later-onLoad");
define("siteab_rating_later_scroll", "Later-onScroll");

function set_siteab_comment_version_cookie() {
  // TODO: automatically figure out domain
 global $siteab_comment_version;
 $expire = time() + 60*60*24*365*2; // 2 years
 setcookie("siteab_comment_version", $siteab_comment_version, $expire, COOKIEPATH, COOKIE_DOMAIN);
}

function set_siteab_rating_version_cookie() {
  // TODO: automatically figure out domain
 global $siteab_rating_version;
 $expire = time() + 60*60*24*365*2; // 2 years
 setcookie("siteab_rating_version", $siteab_rating_version, $expire, COOKIEPATH, COOKIE_DOMAIN);
}

function set_siteab_userid_cookie() {
  // TODO: automatically figure out domain
 global $siteab_userid;
 $expire = time() + 60*60*24*365*2; // 2 years
 $siteab_userid = get_option('siteab_next_user_id');
 update_option('siteab_next_user_id', get_option('siteab_next_user_id') + 1);
 setcookie("siteab_userid", $siteab_userid, $expire, COOKIEPATH, COOKIE_DOMAIN);
}

function choose_versions() {
  global $siteab_comment_version;
  global $siteab_rating_version;
  global $siteab_userid;

  // ** Condition of the Comment experiment
  // If the user already has been assigned a version, use it
  if (isset($_COOKIE["siteab_comment_version"])) {
    $siteab_comment_version = $_COOKIE["siteab_comment_version"];
  } else { // If not, then choose a condition for them
    $vers = rand(0,5);
    switch($vers) {
      case 0: $siteab_comment_version = siteab_comment_noask; break;
      case 1: $siteab_comment_version = siteab_comment_com_load; break;
      case 2: $siteab_comment_version = siteab_comment_com_scroll; break;
      case 3: $siteab_comment_version = siteab_comment_later_load; break;
      case 4: $siteab_comment_version = siteab_comment_later_scroll; break;
    }
    add_action('send_headers', 'set_siteab_comment_version_cookie');
    siteab_debug('Assigned Comment version' . $siteab_comment_version);
  }
  // ** Condition of the Rating experiment
  // If the user already has been assigned a version, use it
  if (isset($_COOKIE["siteab_rating_version"])) {
    $siteab_rating_version = $_COOKIE["siteab_rating_version"];
  } else { // If not, then choose a condition for them
    $vers = rand(0,5);
    switch($vers) {
      case 0: $siteab_rating_version = siteab_rating_noask; break;
      case 1: $siteab_rating_version = siteab_rating_com_load; break;
      case 2: $siteab_rating_version = siteab_rating_com_scroll; break;
      case 3: $siteab_rating_version = siteab_rating_later_load; break;
      case 4: $siteab_rating_version = siteab_rating_later_scroll; break;
    }
    add_action('send_headers', 'set_siteab_rating_version_cookie');
    siteab_debug('Assigned Rating version' . $siteab_rating_version);
  }
  if (isset($_COOKIE["siteab_userid"])) {
    $siteab_userid = $_COOKIE["siteab_userid"];
  } else {
    add_action('send_headers', 'set_siteab_userid_cookie');
    siteab_debug('Assigning new userid');
  }
  if ((get_option('siteab_experiment') == 'comment') && 
      (get_option('siteab_debug_force_condition') == 1)) {
    $siteab_comment_version = get_option('siteab_debug_condition');
  }
  if ((get_option('siteab_experiment') == 'rating') && 
      (get_option('siteab_debug_force_condition') == 1)) {
    $siteab_rating_version = get_option('siteab_debug_condition');
  }
  switch(get_option('siteab_experiment')) {
    case 'comment': siteab_debug('Condition: ' . $siteab_comment_version); break;
    case 'rating': siteab_debug('Condition: ' . $siteab_rating_version); break;
    default: siteab_debug('Unknown condition'); break;
  }
}

if (!is_admin()) {
  add_action('init', 'choose_versions');
}

/** Append logging info to all page loads **/

function siteab_log($action, $postid = 0, $commentid = 0) {
  global $siteab_rating_version;
  global $siteab_comment_version;
  global $siteab_userid;
  global $wpdb;

  $log_table_name = $wpdb->prefix . "siteab_log";
  
  $siteab_experiment = get_option('siteab_experiment');
  
  switch($siteab_experiment) {
    case 'comment': $siteab_version = $siteab_comment_version; break;
    case 'rating': $siteab_version = $siteab_rating_version; break;
    default: $siteab_version = 'disabled';
  }
  
  $wpdb->insert($log_table_name, array('experiment' => $siteab_experiment, 'cond' => $siteab_version, 'action' => $action, 'userid' => $siteab_userid, 'postid' => $postid, 'commentid' => $commentid), array('%s', '%s', '%s', '%d', '%d', '%d'));
}

function siteab_log_pageview() {
  global $post;

  $postid = 0;
  if (is_single()) {
    $postid = $post->ID;
  }
  
  // Actually add the log entry
  siteab_log($_SERVER['REQUEST_URI'], $postid);

  if (get_option(siteab_debug) == 'enabled') {  // Have to do this by hand since we are already in wp_footer
	echo("Logged Pageview");
  }
}
if (!is_admin()) {
  add_action('wp_footer', 'siteab_log_pageview');
}

/*********
 Administration code
 - Adds options menu
 - Displays options page, where this experiment can be enabled
 *********/
 
if ( is_admin() ) {
  add_action('admin_menu', 'siteab_menu');
  add_action('admin_init', 'register_siteab_settings');  
}

function siteab_menu() {
	  add_options_page('SiteAB Options', 'SiteAB Options', 'administrator', __FILE__, 'siteab_options');
}

function register_siteab_settings() {
  register_setting('siteab_options', 'siteab_experiment');
  register_setting('siteab_options', 'siteab_debug');
  register_setting('siteab_options', 'siteab_debug_force_condition');
  register_setting('siteab_options', 'siteab_debug_condition');
  register_setting('siteab_options', 'siteab_enable_comment_ratings');
}

function siteab_options() {
?>
<div class="wrap">
<h2> Site AB Testing Options </h2>

<form method="post" action="options.php">

<p> Number of User IDs assigned: <?php echo get_option('siteab_next_user_id')-1; ?> </p>
<p> Current Database version: <?php echo get_option('siteab_db_version'); ?> </p>
<span> Which experiment should be run? </span>
<ul>
<li> <input type="radio" name="siteab_experiment" value="comment" <?php if (get_option('siteab_experiment') == 'comment') { echo "checked"; } ?> />
Comment Experiment </li>
<li> <input type="radio" name="siteab_experiment" value="rating" <?php if (get_option('siteab_experiment') == 'rating') { echo "checked"; } ?> /> 
Rating Experiment </li>
<li> <input type="radio" name="siteab_experiment" value="disabled" <?php if (get_option('siteab_experiment') == 'disabled') { echo "checked"; } ?> />
Disabled; no experiment running </li>
</ul>
<p> &nbsp; </p>
<p> At the end of the experiment, we need people to rate all of the comments that were contributed.   This switch turns on this ability. </p>
<span> Should logged-in users be able to rate comments? </span>
<ul>
<li> <input type="radio" name="siteab_enable_comment_ratings" value="1" <?php if (get_option('siteab_enable_comment_ratings') == 1) { echo 'checked'; } ?> /> 
Turn on comment ratings</li>
<li> <input type="radio" name="siteab_enable_comment_ratings" value="0" <?php if (get_option('siteab_enable_comment_ratings') == 0) { echo 'checked'; } ?> /> 
Turn off comment ratings</li>
</ul>
<p> &nbsp; </p>
<p> This plugin also has a debugging mode, for viewing extra behind-the-scenes information and forcing everyone into specific conditions. </p>
<span> Should debugging information be printed out? </span>
<ul>
<li> <input type="radio" name="siteab_debug" value="enabled" <?php if (get_option('siteab_debug') == 'enabled') { echo "checked"; } ?> /> 
Enable Debugging </li>
<li> <input type="radio" name="siteab_debug" value="disabled" <?php if (get_option('siteab_debug') == 'disabled') { echo "checked"; } ?> />
Disable Debugging </li>
</ul>
<?php  if (get_option('siteab_debug') == 'enabled') { ?>
<span>Force a particular condition on all users?</span>
<ul>
<li> <input type="radio" name="siteab_debug_force_condition" value="1" <?php if (get_option('siteab_debug_force_condition') == 1) { echo "checked"; } ?> /> 
Everyone gets the same condition </li>
<li> <input type="radio" name="siteab_debug_force_condition" value="0" <?php if (get_option('siteab_debug_force_condition') == 0) { echo "checked"; } ?> />
Everyone gets their normal condition</li>
</ul>
  <?php if (get_option('siteab_experiment') == 'comment') { ?>
  <span> Which condition should be forced upon people? </span>
  <ul>
  <li> <input type="radio" name="siteab_debug_condition" value="<?php echo siteab_comment_noask; ?>" <?php if (get_option('siteab_debug_condition') == siteab_comment_noask) { echo 'checked'; } ?> />
  No Ask </li>
  <li> <input type="radio" name="siteab_debug_condition" value="<?php echo siteab_comment_com_load; ?>" <?php if (get_option('siteab_debug_condition') == siteab_comment_com_load) { echo 'checked'; } ?> />
  Comment Immediately upon loading full page </li>
  <li> <input type="radio" name="siteab_debug_condition" value="<?php echo siteab_comment_com_scroll; ?>" <?php if (get_option('siteab_debug_condition') == siteab_comment_com_scroll) { echo 'checked'; } ?> />
  Comment Immediately upon scrolling to the end of the story </li>
  <li> <input type="radio" name="siteab_debug_condition" value="<?php echo siteab_comment_later_load; ?>" <?php if (get_option('siteab_debug_condition') == siteab_comment_later_load) { echo 'checked'; } ?> />
  Comment Later upon loading the page </li>
  <li> <input type="radio" name="siteab_debug_condition" value="<?php echo siteab_comment_later_scroll; ?>" <?php if (get_option('siteab_debug_condition') == siteab_comment_later_scroll) { echo 'checked'; } ?> />
  Comment Later upon scrolling to the end of the story </li>
  </ul>
  <?php } ?>
<?php } ?>
<?php settings_fields('siteab_options'); ?>
<p class="submit">
<input type="submit" class="button-primary" value="<?php _e('Save Changes'); ?>" />
</p>
</form>
</div>
<?php 
}

/**********
 Common, shared code across the conditions
 **********/
function siteab_display_rating($name, $action, $rating = 0, $extras = array() ) {
  $out = '<form action="' . admin_url("admin-ajax.php") . '" method="post" id="rating_' . $id . '" class="rating_form">
  <input name="action" type="hidden" value="' . $action . '" />';
  foreach ($extras as $extra_name => $extra_value) {
     $out .= '<input name="' . $extra_name .'" type="hidden" value="' . $extra_value .'" />';
  }
  $out .= '<input name="' . $name . '" type="radio" class="myrating" value="1"';
  if ($rating == 1) { $out .= ' checked'; }
  $out .= ' /><input name="' . $name . '" type="radio" class="myrating" value="2"';
  if ($rating == 2) { $out .= ' checked'; }
  $out .= ' /><input name="' . $name . '" type="radio" class="myrating" value="3"';
  if ($rating == 3) { $out .= ' checked'; }
  $out .= ' /><input name="' . $name . '" type="radio" class="myrating" value="4"';
  if ($rating == 4) { $out .= ' checked'; }
  $out .= ' /><input name="' . $name . '" type="radio" class="myrating" value="5"';
  if ($rating == 5) { $out .= ' checked'; }
  $out .= ' />
  </form>';
  return $out;
}


/**********
 Code for the 5-star rating system
 - Loads jquery and jquery-rating javascripts on every page
 - Appends star ratings display to every post
 **********/

if ((get_option('siteab_experiment') == 'rating') && (!is_admin())) {
  function siteab_rating_init_header() {
	// Tell wordpress to load the jquery form plugin
	wp_enqueue_script('jquery-form');
	// Tell wordpress to load both jquery and the jquery.rating plugin scripts
	wp_enqueue_script('jquery.rating', 
	  WP_PLUGIN_URL . '/siteab/star/jquery.rating.js',
	  array('jquery'));
	// Tell wordpress to load my custom javascript
	wp_enqueue_script('siteab-rating',
	  WP_PLUGIN_URL . '/siteab/siteab-rating.js',
	  array('jquery', 'jquery.rating', 'jquery-form'));
  }
  function siteab_rating_init_css() {
	echo '<link type="text/css" rel="stylesheet" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/siteab/star/jquery.rating.css" />';
  }
  if (!is_admin()) {
	add_action('init', 'siteab_rating_init_header');
	add_action('wp_head', 'siteab_rating_init_css');
  }  
  // Append star ratings to every post
  function siteab_append_ratings($content) {
	global $id;
	return $content . siteab_display_rating("myrating", $id);
  }
  add_filter("the_content", "siteab_append_ratings");
  
  function siteab_record_rating() {
	global $wpdb;
	global $siteab_version;
	$post_id = $_POST['postid'];
	$user_id = $_COOKIE['siteab_userid'];
  //  $user_id = 1;
	$rating = $_POST['myrating'];
	$cond = $siteab_version;
	$rating_table_name = $wpdb->prefix . "siteab_ratings";
	$ins_sql = $wpdb->prepare('INSERT INTO ' . $rating_table_name . ' (userid, postid, cond, rating) VALUES (%d, %d, %s, %d);', $user_id, $post_id, $cond, $rating);
	$out = $wpdb->query($ins_sql);  
	if ($out === FALSE) {
	  die("Error with MySQL query: " . $ins_sql);
	}
	echo "huzzah!";
	exit;
  }
  add_action('wp_ajax_siteab_post_rating', 'siteab_record_rating');
  add_action('wp_ajax_nopriv_siteab_post_rating', 'siteab_record_rating');
} /* siteab_rating_exp_enabled */


/**********
 Code for the "power of the ask" comment experiment
 - Load jquery and jquery UI dialog plugins javascript & CSS
 - Load custom javascript file
 - Display popups at appropriate time depending on condition
 **********/
if ((get_option('siteab_experiment') == 'comment') && (!is_admin())) {
  global $override_dialog;
  $override_dialog = false;
  require_once('new-comment.php');
  function siteab_comment_init_header() {
	// Tell wordpress to load the jquery form plugin
	wp_enqueue_script('jquery-form');
	// Load the jquery UI bundle
	wp_enqueue_script('jquery-ui-custom', WP_PLUGIN_URL . '/siteab/jquery-ui/js/jquery-ui-1.7.2.custom.min.js',
	array('jquery'), '1.7.2');
	wp_enqueue_script('jquery.rating', 
	  WP_PLUGIN_URL . '/siteab/star/jquery.rating.js',
	  array('jquery'));
	// Load my own script
	wp_enqueue_script('siteab-comment',
	  WP_PLUGIN_URL . '/siteab/siteab-comment.js',
	  array('jquery', 'jquery-form', 'jquery-ui-custom'));
  }
  function siteab_comment_init_css() {
	echo '<link type="text/css" rel="stylesheet" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/siteab/jquery-ui/css/smoothness/jquery-ui-1.7.2.custom.css" />';
	echo '<link type="text/css" rel="stylesheet" href="' . get_bloginfo('wpurl') . '/wp-content/plugins/siteab/star/jquery.rating.css" />';
  }
  add_action('init', 'siteab_comment_init_header');
  add_action('wp_head', 'siteab_comment_init_css');  
  function siteab_comment_display_popup($content) {
    global $id; // The ID of the current post
    global $siteab_comment_version;
    global $override_dialog;
    if ((is_single()) && ($siteab_comment_version != siteab_comment_noask) && (!$override_dialog)) {
      siteab_debug("Displaying dialog");
      switch($siteab_comment_version) {
        case siteab_comment_com_load:
          $html_class = 'dialog_onload';
          $dialog = siteab_comment_form(get_post($id), true, $html_class);
          break;
        case siteab_comment_com_scroll:
          $html_class = 'dialog_onscroll';
          $dialog = siteab_comment_form(get_post($id), true, $html_class);
          break;
        case siteab_comment_later_load:
          $html_class = 'dialog_onload';
          $dialog = siteab_later_form(get_post($id), true, $html_class);
          break;
        case siteab_comment_later_scroll:
          $html_class = 'dialog_onscroll';
          $dialog = siteab_later_form(get_post($id), true, $html_class);
          break;
      }
      return $content . $dialog; 
    } else {
      return $content;
    }
  }
  add_filter('the_content', 'siteab_comment_display_popup');
  // Log when a comment is posted
  function siteab_log_comment($comment_id, $status) {
    global $override_dialog;
    $comment = get_comment($comment_id);
    $postid = $comment->comment_post_ID;
    $msg = "Added Comment";
    if (defined($_POST['comment_dialog'])) {
      $msg .= ' from dialog';
    }
    siteab_log($msg, $postid, $comment_id);
    $override_dialog = true;
  }
  add_action('comment_post', 'siteab_log_comment');
  // Don't ask for a comment if the user has already posted one
  function siteab_override_comment() {
    global $wpdb;
    global $siteab_userid;
    global $override_dialog;
    global $post;
    $log_table_name = $wpdb->prefix . "siteab_log";
    
    siteab_debug("Checking if the user has posted a comment");
    $sql = $wpdb->prepare('select id from ' . $log_table_name . ' where userid = %d and postid = %d and action like %s', $siteab_userid, $post->ID, "Added Comment%");
    siteab_debug($sql);
    $res = $wpdb->query($sql);
    if ($wpdb->num_rows > 0) {
      siteab_debug("Overriding the display of a dialog.");
      $override_dialog = true;
    }
  }
  add_action('wp_head', 'siteab_override_comment');
  function siteab_id_in($id, $list) {
    foreach ($list as $list_id) {
      if ($list_id == $id) return true;
    }
    return false;
  }
  function siteab_filter_comments($comment_array) {
    global $wpdb;
    global $siteab_comment_version;
    global $siteab_experiment;
    global $post;

    if (is_user_logged_in()) { return $comment_array; }
        
    $postid = $post->ID;
    
    $siteab_experiment = get_option('siteab_experiment');

    $log_table_name = $wpdb->prefix . "siteab_log";
    $query = $wpdb->prepare('select commentid from ' . $log_table_name . ' where postid = %d and cond = %s and experiment = %s and action like %s', $postid, $siteab_comment_version, $siteab_experiment, 'Added Comment%');
    $cond_id_list = $wpdb->get_col($query);  // Retrieve the list of which comment IDs to display
    // Now, filter the main comment array by this list of comment IDs
    $new_comment_array = array();
    foreach ($comment_array as $comment) {
      if (siteab_id_in($comment->comment_ID, $cond_id_list)) {
        $new_comment_array[] = $comment;
      }
    }
    return $new_comment_array;
  }
  function siteab_filter_comments_number($number) {
    global $wpdb;
    global $siteab_comment_version;
    global $siteab_experiment;
    global $post;
    
    if (is_user_logged_in()) { return $number; }
    
    $postid = $post->ID;
    
    $siteab_experiment = get_option('siteab_experiment');

    $log_table_name = $wpdb->prefix . "siteab_log";
    $query = $wpdb->prepare('select count(distinct commentid) from ' . $log_table_name . ' where postid = %d and cond = %s and experiment = %s and action like %s', $postid, $siteab_comment_version, $siteab_experiment, 'Added Comment%');
    siteab_debug($query);
    $new_number = $wpdb->get_var($query);  // Retrieve the list of which comment IDs to display
    return $new_number;
  }
  add_filter('comments_array', 'siteab_filter_comments');
  add_filter('get_comments_number', 'siteab_filter_comments_number');
}
if ((get_option('siteab_experiment') == 'comment')) {
  function siteab_get_comment_condition($postid, $commentid) {
    global $wpdb;
    
    $siteab_experiment = get_option('siteab_experiment');
    $log_table_name = $wpdb->prefix . "siteab_log";
    
    $query = $wpdb->prepare('select cond from ' . $log_table_name . ' where postid = %d and commentid = %s and experiment = %s and action like %s', $postid, $commentid, $siteab_experiment, 'Added Comment%');
    siteab_debug($query);
    $cond = $wpdb->get_var($query);  // Retrieve the list of which comment IDs to display
    
    return $cond;
  }
  function siteab_display_condition($cond) {
    return '<p><span style="font-weight: bold;">Condition: </span> ' . $cond . '</p>';
  }
  function siteab_append_comment_rating($comment_text) {
    global $comment;
    global $wpdb;
    global $current_user;
    global $siteab_comment_version;
    
    if (!is_user_logged_in()) {
      return $comment_text;
    }
    
    get_currentuserinfo();
    
    $comment_rating_table_name = $wpdb->prefix . "siteab_comment_ratings";
    $current_rating_sql = $wpdb->prepare('select rating from ' . $comment_rating_table_name . ' where postid = %d and commentid = %d and userid = %d limit 1', $comment->comment_post_ID, $comment->comment_ID, $current_user->ID);
    $cur_rating = $wpdb->get_var($current_rating_sql);
    
    $cond = siteab_get_comment_condition($comment->comment_post_ID, $comment->comment_ID);
    
    return $comment_text . 
      siteab_display_condition($cond) . 
      siteab_display_rating("rating-" . $comment->comment_ID, 
        'siteab_comment_rating', 
        $cur_rating,
		array("postid" => $comment->comment_post_ID, 
			  "commentid" => $comment->comment_ID,
			  "cond" => $cond));
  }
  function siteab_comment_record_rating() {
    global $current_user;
    global $wpdb;
        
    get_currentuserinfo();
    
    $userid = $current_user->ID;
    $postid = $_POST['postid'];
    $commentid = $_POST['commentid'];
    $cond = $_POST['cond'];
    $rating = $_POST['rating-' . $commentid];
    
	$comment_rating_table_name = $wpdb->prefix . "siteab_comment_ratings";
	$ins_sql = $wpdb->prepare('INSERT INTO ' . $comment_rating_table_name . ' (userid, postid, commentid, cond, rating) VALUES (%d, %d, %d, %s, %d);', $userid, $postid, $commentid, $cond, $rating);
	$out = $wpdb->query($ins_sql);  
	if ($out === FALSE) {
	  die("Error with MySQL query: " . $ins_sql);
	}
	echo "huzzah!";
	exit;    
  }
  if (get_option('siteab_enable_comment_ratings') == 1) {
    add_filter('comment_text', 'siteab_append_comment_rating');
  }
  add_action('wp_ajax_siteab_comment_rating', 'siteab_comment_record_rating');
  add_action('wp_ajax_nopriv_siteab_comment_rating', 'siteab_comment_record_rating'); 
}

// TODO: In the "comment added" log, note if no dialog was displayed.
// TODO: Implement "rating" experiment

?>


